import on from '@thinkeloquent/rd2020-dom/src/element/on';
import addClass from '@thinkeloquent/rd2020-dom/src/element/addClass';
import removeClass from '@thinkeloquent/rd2020-dom/src/element/removeClass';
import getTargetFromEvent from '@thinkeloquent/rd2020-dom/src/element/getTargetFromEvent';
import { findWithDocument } from '@thinkeloquent/rd2020-dom/src/selector/find';
import ElementBoxSelector from '@thinkeloquent/rd2020-element-box-selector/dist/element-box-selector.umd.bundle';

const outlineClassName = 'b72e1ee-2736-4254-aacf-1540f0f86777';
const outlineSelectedClassName = `${outlineClassName}-selected`;
const headStyleTagsId = 'd8ba4-435a-b684-c5f48823d7a2';

const headStyleTags = `
<style id="${headStyleTagsId}">
    .${outlineClassName} {
        outline: 2px solid red;
    }
    .${outlineSelectedClassName}{
        outline:2px solid blue;
    }
</style>
`;

class ElementHighlight {
  constructor() {
    this.previousHoveredElement = null;
    this.previousSelectedElement = null;
  }

  attachHeadStyles(documentHead) {
    documentHead.insertAdjacentHTML('beforeend', headStyleTags);
  }
  
  cleanup() {
    removePreviousSelection();
  }

  attachBodyListeners(documentBody, options = {}) {
    let disposeOf;
    const { onClick, onBodyClick, onSelectedElementClick, canEditable } = options;
    const hasOnClick = typeof(onClick) === 'function';
    const hasOnBodyClick = typeof(onBodyClick) === 'function';
    const hasOnSelectedElementClick = typeof(onSelectedElementClick) === 'function';

    if(typeof(canEditable) !== 'function') {
      throw new Error('MISSING_FUNCTION unEditable')
    }

    const removePreviousSelection = () => {
      let currentlySelected = findWithDocument(documentBody, `.${outlineSelectedClassName}`);
      if (currentlySelected && currentlySelected.length) {
          removeClass(currentlySelected, outlineSelectedClassName);
      }
      return currentlySelected;
    };

    const disposeOfMouseOver = on(documentBody, 'mouseover', (event) => {
        addClass(event.target, outlineClassName);
        this.previousHoveredElement = event.target;
    });

    const disposeOfMouseOut = on(documentBody, 'mouseout', (event) => {
        const el = event ? event.target : this.previousHoveredElement;
        if (el) removeClass(el, outlineClassName);
    });

    const disposeOfClick = on(documentBody, 'click', (event) => {
      event.preventDefault();
      event.stopPropagation();
      const target = getTargetFromEvent(event);

      if(!canEditable(target)) {
        console.log('can not edit', target)
        return;
      }

      if(this.previousSelectedElement === target) {
        if(hasOnSelectedElementClick) onSelectedElementClick(target);
        return;
      }

      const tagName = String(target.tagName).toLowerCase();
      const unEditable = ['body', 'html'].indexOf(tagName) !== -1;      
      const previousSelectedElement = removePreviousSelection();

      if(unEditable) {
        if(hasOnBodyClick) onBodyClick(target, previousSelectedElement);
        return;
      }

      if(hasOnClick) onClick(target, previousSelectedElement);
      addClass(target, outlineSelectedClassName);
      this.previousSelectedElement = target;
    });

    disposeOf = () => {
      disposeOfMouseOver();
      disposeOfMouseOut();
      disposeOfClick();
    };

    return disposeOf;
  }

  getData() {
    return {
      dom: { ...this.dom },
      draft: { ...this.draft },
      originalText: { ...this.originalText },
    }
  }

}

export default ElementHighlight;
export { ElementBoxSelector }